"use strict";
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const child_process_1 = require("child_process");
const fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
const klawSync = require("klaw-sync");
const path_1 = tslib_1.__importDefault(require("path"));
const process_1 = tslib_1.__importDefault(require("process"));
// Some constants.
const CWD = process_1.default.cwd();
const SETTINGS_FILE_NAME = 'readme-settings.json';
const SETTINGS_FILE_PATH = path_1.default.resolve(CWD, SETTINGS_FILE_NAME);
class ReadmeUtils extends command_1.Command {
    async run() {
        const { flags } = this.parse(ReadmeUtils);
        if (flags.init) {
            await this.initRun(flags.name);
        }
        else if (flags.build) {
            await this.buildRun();
        }
    }
    async initRun(name) {
        const projectDir = name ? name : CWD;
        this.debug(`Attempting to create project inside the ${projectDir} folder`);
        // TODO: Verify directory exists, if it doesn't create it.
        await fs_extra_1.default.ensureDir(projectDir);
        // TODO: Copy files to project folder
        await fs_extra_1.default.copy(path_1.default.resolve(__dirname, '..', 'template'), projectDir);
        const settingsPath = path_1.default.resolve(projectDir, SETTINGS_FILE_NAME);
        const settings = await fs_extra_1.default.readJSON(settingsPath);
        settings.project = path_1.default.basename(projectDir);
        await fs_extra_1.default.writeJSON(settingsPath, settings);
        this.exit(0);
    }
    /**
     * Reads the project's settings.
     */
    async readSettings() {
        this.debug('Reading settings...');
        // Check settings file exist
        const settingsExist = await fs_extra_1.default.pathExists(SETTINGS_FILE_PATH);
        if (!settingsExist) {
            throw Error(`Missing settings file ${SETTINGS_FILE_PATH}.`);
        }
        // Read settings file
        const settingsData = await fs_extra_1.default.readFile(SETTINGS_FILE_PATH);
        const settings = JSON.parse(settingsData.toString('utf8'));
        return settings;
    }
    /**
     * Creates pandoc parameters from the specified settings.
     * @param settings Build settings.
     */
    async buildParameters(settings) {
        this.debug('Creating pandoc parameters...');
        // Check if source directory was overrode, set default if it wasn't
        settings.sourceDir = settings.sourceDir ? settings.sourceDir : 'src';
        settings.sourceDir = path_1.default.resolve(CWD, settings.sourceDir);
        // Get all the input files
        const inputFiles = klawSync(settings.sourceDir, { nodir: true });
        // Check there's at least one input file
        if (inputFiles.length === 0) {
            throw Error('Empty source directory.');
        }
        // Create minimal pandoc parameters
        const pandoc = settings.pandoc ? settings.pandoc : 'pandoc';
        const input = inputFiles.map(s => path_1.default.relative(CWD, s.path)).join(' ');
        const output = `--output=${settings.output}`;
        const parameters = {
            input,
            pandoc,
            output,
        };
        // Check if metadata is specified and verifies it exists
        // if it does, update parameters
        if (settings.metadata) {
            parameters.input = `${settings.metadata} ${parameters.input}`;
        }
        // Check if bibliography is specified and verifies it exists
        // if it does, update parameters
        if (settings.bib) {
            parameters.bibliography = `--bibliography=${settings.bib}`;
        }
        // Check if header is specified and verifies it exists
        // if it does, update parameters
        if (settings.header) {
            parameters.header = `--include-in-header=${settings.header}`;
        }
        return parameters;
    }
    /**
     * Creates a pandoc build command with the specified parameters.
     * @param parameters pandoc parameters
     */
    async buildCommand(parameters) {
        const options = [];
        if (parameters.bibliography) {
            options.push(parameters.bibliography);
        }
        if (parameters.header) {
            options.push(parameters.header);
        }
        options.push(parameters.input);
        options.push(parameters.output);
        return `${parameters.pandoc} ${options.join(' ')}`;
    }
    /**
     * Reads user settings, constructs the corresponding pandoc command and builds the project.
     */
    async buildRun() {
        const settings = await this.readSettings();
        const parameters = await this.buildParameters(settings);
        const pandoc = await this.buildCommand(parameters);
        this.log(`Running build command:\n> ${pandoc}`);
        child_process_1.exec(pandoc, (error, stdout, sterr) => {
            if (error) {
                const { code } = error;
                if (code !== 0 && code !== undefined) {
                    this.warn(`Received non-zero (${code}) exit status from pandoc.`);
                    this.error(sterr.trim(), { code: code.toString(), exit: false });
                }
            }
            else if (stdout.length > 0)
                this.log(stdout);
        });
    }
}
ReadmeUtils.description = 'A thin wrapper that enables external configuration for the pandoc command.';
ReadmeUtils.flags = {
    build: command_1.flags.boolean({
        char: 'b',
        description: 'if the project should be built.',
        default: true,
    }),
    init: command_1.flags.boolean({
        char: 'i',
        description: 'generate starter project.',
        exclusive: ['build'],
    }),
    name: command_1.flags.string({
        char: 'n',
        description: 'project name and folder, defaults to the current working directory.',
        dependsOn: ['init'],
        exclusive: ['build'],
    }),
};
module.exports = ReadmeUtils;
